<?php
namespace Teknisa\SUP\Controllers;

use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\SUP\Services\SupplyTekApprovalWorkflowService;
use Teknisa\SUP\Controllers\SendEmailController;

/**
 * Handles Approval/Disapproval Workflow from SupplyTek
 * 
 * Controls the actions needed for each workflow and send
 * emails to users with notifications of the status being
 * updated.
 *
 * @author Rômulo Alves Lousada
 */
class SupplyTekApprovalWorkflowController
{
    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param SupplyTekApprovalWorkflowService $service Object to access SupplyTekApprovalWorkflowService
     * @param SendEmailController $sendEmail Object to access SendEmailController
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function __construct(SupplyTekApprovalWorkflowService $service, SendEmailController $sendEmail)
    {
        $this->service = $service;
        $this->sendEmail = $sendEmail;
    }
    
    /**************************************************************************/
    /*                             WORKFLOW START                             */
    /**************************************************************************/
    
    /**
     * Initial function to start the workflow process
     * 
     * @param array $row Array that receives data from the caller
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function startWorkflow($row)
    {
        try {
            $this->service->beginTransaction();
            
            if ($row instanceof Row) {
                $row = $row->getRow();
                $this->service->setEnvironment($row);
            }
            
            $this->service->saveStatus($row);
            
            switch ($row['STATUS']) {
                case 'A':
                    $this->startApprovalWorkflow($row);
                break;
                
                case 'R':
                    $this->startDisapprovalWorkflow($row);
                break;
            }
            
            $this->service->commitTransaction();
        } catch (Exception $e) {
            $this->service->rollbackTransaction();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**************************************************************************/
    /*                          DISAPPROVAL WORKFLOW                          */
    /**************************************************************************/
    
    /**
     * Main function related to Disapproval workflow
     * 
     * Calls the functions related to disapproval workflow and
     * retrieve email data to send email after the process.
     * 
     * @param array $row Array with data received to start the process
     * 
     * @return void
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function startDisapprovalWorkflow($row)
    {
        try {
            $this->service->updateAFDisapproval($row);
            
            $data = $this->getDataDisapproval($row);
            $this->service->saveNotificationDisapproval($row, $data);
            
            $messages = $this->createMessagesDisapproval($row, $data);
            $this->sendEmail->sendMailSupplyTek($messages);
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    /**
     * Return data needed on disapproval process
     * 
     * @param array $row Array with data received to start the process
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function getDataDisapproval($row)
    {
        try {
            $data = array();
            $data['AF'] = $this->service->getAFDataDisapproval($row);
            $data['USER'] = $this->service->getUserDataDisapproval($row);
            
            return $data;
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    /**
     * Creates an array with data needed on the mail process of disapproval
     * 
     * @param array $row Array with data received to start the process
     * @param array $data Array with data needed on the process
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function createMessagesDisapproval($row, $data)
    {
        try {
            $messages = array();
    
            $messages["RECIPIENT"] = $data["USER"][0]["DSEMAILOPER"];
            
            $messages["SUBJECT"] = "Reprovação da AF - Nº " . $row["NRAUTOFORN"];
            
            $messages["CONTENT"] = "<span>
                                        Prezado(a) " . $data["AF"][0]["NMDESTINATARIO"] . ",
                                    </span>
                            		<br/><br/>
                            		<span>
                            		    O seu pedido de nº " . $row["NRAUTOFORN"] . " criado no dia " . $data["AF"][0]["DTAUTOFORN"] . " 
                            		    infelizmente foi reprovado por " . $data["USER"][0]["NMREPROV"] . " dia " . date("d/m/Y") . ".
                            		</span>
                            		<br/><br/>
                            		<span>
                            		    Motivo da reprovação: <b>" . $row["DSJUSTIFIC"] . "</b>
                            		</span>";
                            		
            return $messages;
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    /**************************************************************************/
    /*                            APPROVAL WORKFLOW                           */
    /**************************************************************************/
    
    /**
     * Main function related to Approval workflow
     * 
     * Calls the functions related to next approver or finish the approval
     * process, depending if it has more approvers or is the last/only one.
     * 
     * @param array $row Array with data received to start the process
     * 
     * @return void
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function startApprovalWorkflow($row)
    {
        try {
            $row["NRCONFIGALCADA"] = $this->service->getNrConfigAlcada($row);
            
            $data = $this->service->getDataApproval($row);
            $approver = $this->service->searchForNextApprover($row);
            
            if ($approver && $approver[0]) {
                $this->startNextApproverWorkflow($row, $data[0], $approver[0]);
            } else {
                $this->finishApprovalWorkflow($row, $data[0]);
            }
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    /*************************************/
    /*         HAVE NEXT APPROVER        */
    /*************************************/
    
    /**
     * Sub function related to Next Approver workflow
     * 
     * Calls the functions related to next approver workflow and
     * create email data needed send email after the process.
     * 
     * @param array $row Array with data received to start the process
     * @param array $data Array with data needed on the process
     * @param array $approver Array with next approver data needed on the process
     * 
     * @return void
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function startNextApproverWorkflow($row, $data, $approver)
    {
        try {
            $this->service->saveNextApproval($row, $approver);
                
            $messages = $this->createMessagesNextApprover($row, $data, $approver);
            $this->sendEmail->sendMailSupplyTek($messages);
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
     /**
     * Creates an array with data needed on the mail process of next approver
     * 
     * @param array $row Array with data received to start the process
     * @param array $data Array with data needed on the process
     * @param array $approver Array with next approver data needed on the process
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function createMessagesNextApprover($row, $data, $approver)
    {
        try {
            $messages = array();
            
            $messages["RECIPIENT"] = $approver["EMAILPROXAPROV"];
            
            $messages["SUBJECT"] = "Aprovação Pendente - Nº " . $row["NRAUTOFORN"];
            
            $messages["CONTENT"] = "<span>
                                    	Prezado(a) " . $approver["NMPROXAPROV"] . ",
                                    </span>
                                    <br/><br/>
                                    <span>
                                    	O pedido de nº " . $row["NRAUTOFORN"] . " 
                                    	da unidade " . $row["CDFILDEST"] . " - " . $data["NMFILDEST"] . " 
                                    	está pendente de sua aprovação no <a href=" . $row['SUPPLYTEK_URL'] . ">SupplyTek</a>.
                                    </span>
                                    <br/><br/>
                                    <span>
                                    	Segue dados do pedido:
                                    </span>
                                    <br/>
                                    <ul>
                                    	<li>Número da AF: " . $row["NRAUTOFORN"] . "</li>
                                    	<li>Unidade: " . $row["CDFILDEST"] . " - " . $data["NMFILDEST"] . "</li>
                                    	<li>Data de Criação: " . $data["DTAUTOFORN"] . "</li>
                                    	<li>Data de Entrega: " . $data["DTPROGENTAF"] . "</li>
                                    	<li>Valor Total da AF: " . $this->formatCurrency($data["VRTOTALAF"]) . "</li>
                                    	<li>Comprador: " . $data["COMPRADOR"] . "</li>
                                    	<li>Forncedor: " . $data["NRINSJURFORN"] . " - " . $data["NMFANTFORN"] . "</li>
                                    	<li>Usuário solicitante: " . $data["NMSOLICITANTE"] . "(" . $data["EMAILSOLICITANTE"] . ")</li>
                                    </ul>";
                            		
            return $messages;
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    /**
     * Received an unformatted value from database and converts to
     * a formatted value with R$.
     * 
     * @param string $currency Value saved on database without proper formatting.
     * 
     * @return string
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function formatCurrency($currency)
    {
        $currency = str_replace(",", ".", $currency);
        $currency = floatval($currency);
        $currency = number_format($currency, 2, ',', '.');
        $currency = "R$ " . $currency;
        
        return $currency;
    }
    
    /*************************************/
    /*      DONT HAVE NEXT APPROVER      */
    /*************************************/
    
    /**
     * Sub function related to Finish Approval workflow
     * 
     * Calls the functions related to finish approval workflow and
     * create email data needed send email after the process.
     * 
     * @param array $row Array with data received to start the process
     * @param array $data Array with data needed on the process
     * 
     * @return void
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function finishApprovalWorkflow($row, $data)
    {
        try {
            $this->service->removeItensWith0Quantity($row);
            
            $nrSeqEnvAF = $this->service->saveSentAFApproval($row, $data);
            
            $this->service->updateAFApproval($row, $nrSeqEnvAF);
            
            $this->service->saveNotificationApproval($row, $data);
            
            $messagesSupplier = $this->createMessagesSupplierApproval($row, $data);
            $this->sendEmail->sendMailSupplyTek($messagesSupplier);
            
            $messagesRequester = $this->createMessagesRequesterApproval($row, $data);
            $this->sendEmail->sendMailSupplyTek($messagesRequester);
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    /**
     * Creates an array with data needed on the mail to the supplier in the process to finish approval
     * 
     * @param array $row Array with data received to start the process
     * @param array $data Array with data needed on the process
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function createMessagesSupplierApproval($row, $data)
    {
        try {
            $messages = array();
            
            $messages["RECIPIENT"] = array_filter(explode(",", $data["DSEMAILFORN"]));
            
            $messages["SUBJECT"] = "Disponibilização de AF - Nº " . $row["NRAUTOFORN"];
            
            $messages["CONTENT"] = "<span>
                                    	Prezado(a) " . $data["NMFANTFORN"] . ",
                                    </span>
                                    <br/><br/>
                                    <span>
                                        Uma Autorização de Fornecimento foi disponibilizada em nosso Portal de Compras.
                                    </span>
                                    <br/><br/>
                                    <span>
                                        Acesse " . $data["DSLINKECOMMERCE"] . " e confira 
                                        o pedido de nº " . $row["NRAUTOFORN"] . " 
                                        da unidade " . $row["CDFILDEST"] . " - " . $data["NMFILDEST"] . ".
                                    </span>
                                    <br/><br/>
                                    <span>
                                    	Não se esqueça de confirmar o atendimento da AF e siga rigorosamente a Autorização de Fornecimento!
                                    </span>
                                    <br/><br/>
                                    <span>
                                        Obrigado.
                                    </span>
                                    <br/>
                                    <span>
                                        " . $data["COMPRADOR"] . " / " . $data["NMSOLICITANTE"] . "
                                    </span>";
                            		
            return $messages;
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    /**
     * Creates an array with data needed on the mail to the requester in the process to finish approval
     * 
     * @param array $row Array with data received to start the process
     * @param array $data Array with data needed on the process
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function createMessagesRequesterApproval($row, $data)
    {
        try {
            $messages = array();
            
            $messages["RECIPIENT"] = $data["EMAILSOLICITANTE"];
            
            $messages["SUBJECT"] = "Aprovação da AF - Nº " . $row["NRAUTOFORN"];
            
            $messages["CONTENT"] = "<span>
                                    	Prezado(a) " . $data["NMSOLICITANTE"] . ",
                                    </span>
                                    <br/><br/>
                                    <span>
                                        O seu pedido de nº " . $row["NRAUTOFORN"] . " 
                                        criado no dia " . $data["DTAUTOFORN"] . " 
                                        foi aprovado por todos os gestores e foi enviado para o fornecedor.
                                    </span>";
                            		
            return $messages;
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
}